package java.util.zip;

import java.nio.charset.CodingErrorAction;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import sun.nio.cs.ArrayEncoder;
import java.nio.charset.CoderResult;
import java.nio.CharBuffer;
import java.nio.ByteBuffer;
import sun.nio.cs.ArrayDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.Charset;

final class ZipCoder
{
    private Charset cs;
    private CharsetDecoder dec;
    private CharsetEncoder enc;
    private boolean isUTF8;
    private ZipCoder utf8;
    
    String toString(byte[] array, int length) {
        CharsetDecoder reset = this.decoder().reset();
        int n = (int)((float)length * reset.maxCharsPerByte());
        char[] array2 = new char[n];
        if (n == 0) {
            return new String(array2);
        }
        if (this.isUTF8 && reset instanceof ArrayDecoder) {
            int decode2 = ((ArrayDecoder)reset).decode(array, 0, length, array2);
            if (decode2 == -1) {
                throw new IllegalArgumentException("MALFORMED");
            }
            return new String(array2, 0, decode2);
        }
        else {
            ByteBuffer wrap = ByteBuffer.wrap(array, 0, length);
            CharBuffer wrap2 = CharBuffer.wrap(array2);
            CoderResult decode = reset.decode(wrap, wrap2, true);
            if (!decode.isUnderflow()) {
                throw new IllegalArgumentException(decode.toString());
            }
            CoderResult flush = reset.flush(wrap2);
            if (!flush.isUnderflow()) {
                throw new IllegalArgumentException(flush.toString());
            }
            return new String(array2, 0, wrap2.position());
        }
    }
    
    String toString(byte[] array) {
        return this.toString(array, array.length);
    }
    
    byte[] getBytes(String s) {
        CharsetEncoder reset = this.encoder().reset();
        char[] charArray = s.toCharArray();
        int n = (int)((float)charArray.length * reset.maxBytesPerChar());
        byte[] original = new byte[n];
        if (n == 0) {
            return original;
        }
        if (this.isUTF8 && reset instanceof ArrayEncoder) {
            int encode2 = ((ArrayEncoder)reset).encode(charArray, 0, charArray.length, original);
            if (encode2 == -1) {
                throw new IllegalArgumentException("MALFORMED");
            }
            return Arrays.copyOf(original, encode2);
        }
        else {
            ByteBuffer wrap = ByteBuffer.wrap(original);
            CoderResult encode = reset.encode(CharBuffer.wrap(charArray), wrap, true);
            if (!encode.isUnderflow()) {
                throw new IllegalArgumentException(encode.toString());
            }
            CoderResult flush = reset.flush(wrap);
            if (!flush.isUnderflow()) {
                throw new IllegalArgumentException(flush.toString());
            }
            if (wrap.position() == original.length) {
                return original;
            }
            return Arrays.copyOf(original, wrap.position());
        }
    }
    
    byte[] getBytesUTF8(String s) {
        if (this.isUTF8) {
            return this.getBytes(s);
        }
        if (this.utf8 == null) {
            this.utf8 = new ZipCoder(StandardCharsets.UTF_8);
        }
        return this.utf8.getBytes(s);
    }
    
    String toStringUTF8(byte[] array, int n) {
        if (this.isUTF8) {
            return this.toString(array, n);
        }
        if (this.utf8 == null) {
            this.utf8 = new ZipCoder(StandardCharsets.UTF_8);
        }
        return this.utf8.toString(array, n);
    }
    
    boolean isUTF8() {
        return this.isUTF8;
    }
    
    private ZipCoder(Charset cs) {
        this.cs = cs;
        this.isUTF8 = cs.name().equals(StandardCharsets.UTF_8.name());
    }
    
    static ZipCoder get(Charset charset) {
        return new ZipCoder(charset);
    }
    
    private CharsetDecoder decoder() {
        if (this.dec == null) {
            this.dec = this.cs.newDecoder().onMalformedInput(CodingErrorAction.REPORT).onUnmappableCharacter(CodingErrorAction.REPORT);
        }
        return this.dec;
    }
    
    private CharsetEncoder encoder() {
        if (this.enc == null) {
            this.enc = this.cs.newEncoder().onMalformedInput(CodingErrorAction.REPORT).onUnmappableCharacter(CodingErrorAction.REPORT);
        }
        return this.enc;
    }
}
